#include <WiFi.h>
#include <HTTPClient.h>
#include <ArduinoJson.h>

const char *ssid = "IRONMAN";
const char *password = "1234567890";

void setup() {
  Serial.begin(115200);
  Serial.println("40s delay for new coad upload check");

  delay(6000);

  connectToWiFi();
}

void loop() {
  getLocation(); // Get location once on startup

  // Deep sleep for 30 seconds
  Serial.println("Entering deep sleep for 30 seconds...");
  esp_sleep_enable_timer_wakeup(30e6); // 30 seconds
  esp_deep_sleep_start();
}

void connectToWiFi() {
  Serial.println("Connecting to WiFi...");
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.println("Connecting...");
  }

  Serial.println("Connected to WiFi");

  // Once connected, get the IP address and use it to get the location
  Serial.print("IP Address: ");
  Serial.println(WiFi.localIP());
  WiFi.setTxPower(WIFI_POWER_MINUS_1dBm); // Set WiFi transmit power to lowest level
}

void getLocation() {
  HTTPClient http;

  // Get the public IP address of the device
  String url = "https://api.ipify.org";
  http.begin(url);

  int httpResponseCode = http.GET();

  if (httpResponseCode == HTTP_CODE_OK) {
    String ipAddress = http.getString();

    // Construct the URL using the public IP address
    String locationUrl = "https://ipapi.co/" + ipAddress + "/json/";

    http.begin(locationUrl);

    int locationResponseCode = http.GET();

    if (locationResponseCode == HTTP_CODE_OK) {
      String payload = http.getString();
      Serial.println(payload);

      // Parse JSON response
      DynamicJsonDocument doc(1024);
      deserializeJson(doc, payload);

      String country = doc["country_name"].as<String>();
      String region = doc["region"].as<String>();
      String city = doc["city"].as<String>();
      double latitude = doc["latitude"].as<double>();
      double longitude = doc["longitude"].as<double>();

      Serial.println("Location:");
      Serial.print("Country: ");
      Serial.println(country);
      Serial.print("Region: ");
      Serial.println(region);
      Serial.print("City: ");
      Serial.println(city);
      Serial.print("Latitude: ");
      Serial.println(latitude, 7);
      Serial.print("Longitude: ");
      Serial.println(longitude, 7);
    } else {
      Serial.print("Error getting location. HTTP error code: ");
      Serial.println(locationResponseCode);
    }
  } else {
    Serial.print("Error getting public IP address. HTTP error code: ");
    Serial.println(httpResponseCode);
  }

  http.end();
}
